
/**
 * @file       EventHandlers.cpp
 * @date       September 13th, 2012
 *
 * @brief      Defines class for plugin event handlers called on some plugin's events like added new source, etc.
 */

// SYSTEM INCLUDES
//
#include <QMessageBox>
#include <QMainWindow>
#include <QDebug>
#include <pqSMAdaptor.h>
#include <pqCoreUtilities.h>
#include <pqActiveObjects.h>
#include <pqApplicationCore.h>
#include <pqDataRepresentation.h>
#include <pqPipelineSource.h>
#include <pqActiveObjects.h>
#include <pqView.h>
#include <pqApplicationCore.h>
#include <pqServerManagerModel.h>
#include <pqPipelineRepresentation.h>
#include <pqObjectBuilder.h>
#include <vtkPVConfig.h>
#include <vtkSmartPointer.h>
#include <vtkSMProxy.h>
#include <vtkSMProperty.h>
#include <vtkSMDoubleVectorProperty.h>
#include <vtkSMStringVectorProperty.h>

// LOCAL INCLUDES
//
#include "EventHandlers.h"

// constructor
EventHandlers::EventHandlers(QObject* ptrParent): QObject(ptrParent)
{
    // initialize object's parameters
    mptrPipelineSource = NULL;
    mptrLinesRepresentation = NULL;
    mptrLabelsPointsRepresentation = NULL;
    mptrDataLabelRepresentation = NULL;

    // connect handlers which should be called when appropriate signals are fired by ServerManager or ObjectBuilder
    QMainWindow* parent_widget = (QMainWindow*)(pqCoreUtilities::mainWidget());
    if(parent_widget != NULL && pqApplicationCore::instance() != NULL && pqApplicationCore::instance()->getObjectBuilder() != NULL)
    {
        connect(pqApplicationCore::instance()->getServerManagerModel(), SIGNAL(sourceAdded(pqPipelineSource*)), this, SLOT(SourceAdded(pqPipelineSource*)));
        connect(pqApplicationCore::instance()->getObjectBuilder(), SIGNAL(dataRepresentationCreated(pqDataRepresentation*)), this, SLOT(DataRepresentationCreated(pqDataRepresentation*)));
    }
}

// destructor
EventHandlers::~EventHandlers()
{
}

// sets data label parameters
void EventHandlers::SetDataLabelParameters(QList<QVariant> rPointLabelColor, int iPointLabelSize, QString strPointLabelFormat, int iPointLabelJustification)
{
    // if data label representation is created, update its attribute name
    if(mptrDataLabelRepresentation != NULL)
    {
        // set new data label parameters
        pqSMAdaptor::setMultipleElementProperty(mptrDataLabelRepresentation->getProxy()->GetProperty("PointLabelColor"), rPointLabelColor);
        pqSMAdaptor::setElementProperty(mptrDataLabelRepresentation->getProxy()->GetProperty("PointLabelFontSize"), iPointLabelSize);
        pqSMAdaptor::setElementProperty(mptrDataLabelRepresentation->getProxy()->GetProperty("PointLabelFormat"), strPointLabelFormat);
        pqSMAdaptor::setElementProperty(mptrDataLabelRepresentation->getProxy()->GetProperty("PointLabelJustification"), iPointLabelJustification);

        // update underlying VTK objects
        mptrDataLabelRepresentation->getProxy()->UpdateVTKObjects();

        // update active view
        pqView* view = pqActiveObjects::instance().activeView();
        if(view != NULL)
            view->render();
    }

    // save data label parameters
    mPointLabelsColor = rPointLabelColor;
    miPointLabelsSize = iPointLabelSize;
    mstrPointLabelFormat = strPointLabelFormat;
    miPointLabelJustification = iPointLabelJustification;
}

// event handler called whenever new pipeline source is added to server manager
void EventHandlers::SourceAdded(pqPipelineSource* ptrPipelineSource)
{
    QMessageBox::about(NULL, "", "SourceAdded");
    // stop catching signal representationAdded fired by server manager model
    disconnect(pqApplicationCore::instance()->getServerManagerModel(), SIGNAL(sourceAdded(pqPipelineSource*)), this, SLOT(SourceAdded(pqPipelineSource*)));

    // catch signals representationAdded fired by current pipeline source
    connect(ptrPipelineSource, SIGNAL(representationAdded(pqPipelineSource*, pqDataRepresentation*, int)), this, SLOT(RepresentationAdded(pqPipelineSource*, pqDataRepresentation*, int)));

    // save reference to added pipeline source
    mptrPipelineSource = ptrPipelineSource;
}

// event handler called whenever new data representation is created
void EventHandlers::RepresentationAdded(pqPipelineSource* ptrPipelineSource, pqDataRepresentation* ptrDataRepresentation, int iSourcePort)
{
    QMessageBox::about(NULL, "", "RepresentationAdded");
    // cache added representation
    if(mptrLinesRepresentation == NULL && iSourcePort == 1)
        // cache lines representation
        mptrLinesRepresentation = ptrDataRepresentation;
    else if(mptrLabelsPointsRepresentation == NULL && iSourcePort == 0)
        // cache labels points representation
        mptrLabelsPointsRepresentation = ptrDataRepresentation;
    else if(mptrDataLabelRepresentation == NULL && iSourcePort == 0)
        // cache labels representation
        mptrDataLabelRepresentation = ptrDataRepresentation;
}

// event handler called whenever new data representation is created by object builder
void EventHandlers::DataRepresentationCreated(pqDataRepresentation* ptrDataRepresentation)
{
    QMessageBox::about(NULL, "", "DataRepresentationCreated");
    // start catching dataRepresentationCreated signal emited by ObjectBuilder
    disconnect(pqApplicationCore::instance()->getObjectBuilder(), SIGNAL(dataRepresentationCreated(pqDataRepresentation*)), this, SLOT(DataRepresentationCreated(pqDataRepresentation*)));

    // stop catching signal and create data label representation
    QMainWindow* parent_widget = (QMainWindow*)(pqCoreUtilities::mainWidget());
    if(parent_widget != NULL && pqApplicationCore::instance() != NULL && pqApplicationCore::instance()->getObjectBuilder() != NULL)
    {
        // get active objects
        pqView* view = ptrDataRepresentation->getView();

        // get object builder
        pqObjectBuilder* builder = pqApplicationCore::instance()->getObjectBuilder();

        // create new representation
        mptrDataLabelRepresentation = builder->createDataRepresentation(mptrPipelineSource->getOutputPort(0), view, "DataLabelRepresentation");
        if(mptrDataLabelRepresentation == NULL)
            return;

        // sets new opacity for label's points
        pqSMAdaptor::setElementProperty(mptrLabelsPointsRepresentation->getProxy()->GetProperty("Opacity"), 0.0);

        // update label's points representation object
        mptrLabelsPointsRepresentation->getProxy()->UpdateVTKObjects();

        // set representation's properties
        pqSMAdaptor::setEnumerationProperty(mptrDataLabelRepresentation->getProxy()->GetProperty("PointFieldDataArrayName"), "Coordinates");
        pqSMAdaptor::setEnumerationProperty(mptrDataLabelRepresentation->getProxy()->GetProperty("PointLabelVisibility"), 1);
        pqSMAdaptor::setMultipleElementProperty(mptrDataLabelRepresentation->getProxy()->GetProperty("PointLabelColor"), mPointLabelsColor);
        pqSMAdaptor::setElementProperty(mptrDataLabelRepresentation->getProxy()->GetProperty("PointLabelFontSize"), miPointLabelsSize);
        pqSMAdaptor::setElementProperty(mptrDataLabelRepresentation->getProxy()->GetProperty("PointLabelFormat"), mstrPointLabelFormat);
        pqSMAdaptor::setElementProperty(mptrDataLabelRepresentation->getProxy()->GetProperty("PointLabelJustification"), miPointLabelJustification);

        // update underlying VTK objects
        mptrDataLabelRepresentation->getProxy()->UpdateVTKObjects();
    }
}
