
/**
 * @file       EventHandlers.h
 * @date       September 13th, 2012
 *
 * @brief      Declares class for plugin event handlers called on some plugin's events like added new source, etc.
 */

#if !defined(EVENT_HANDLERS_H)
#define EVENT_HANDLERS_H

// SYSTEM INCLUDES
//
#include <QObject>
#include <pqPipelineSource.h>
#include <vtkSmartPointer.h>

// USED TYPES
//
class pqRepresentation;
class pqDataRepresentation;

/**
 * @class EventHandlers
 * @brief Defines plugin's event handlers
 * 
 * Class encapsulates handlers called on some plugin's events like added new source, etc. User should only create new instance of this class to use it.
 * 
 * @date    May 14th, 2012
 * @version 1.0
 */
class EventHandlers: public QObject
{

    Q_OBJECT

    //////////////////// PUBLIC ////////////////////

public:

    //==================== EVENT HANDLERS ====================

    /**
     * @brief Constructor
     * Routine is called at the beginning of object's lifecycle
     *
     * @param ptrParent Parent Qt object for plugin starter.
     */
    EventHandlers(QObject* ptrParent = 0);

    /**
     * @brief Destructor
     * Routine is called at the end of object's lifecycle
     */
    ~EventHandlers();

    //////////////////// MUTATORS ////////////////////

    /**
     * @brief 
     * 
     *
     * @param rPointLabelColor New value for per-point label color
     * @param iPointLabelSize New value for per-point label size
     * @param iPointLabelFormat New value for per-point label format
     * @param iPointLabelJustification New value for per-point label justification
     */
    void SetDataLabelParameters(QList<QVariant> rPointLabelColor, int iPointLabelSize, QString strPointLabelFormat, int iPointLabelJustification);

    //////////////////// PRIVATE SLOTS ////////////////////

private slots:

    //==================== PRIVATE SLOT ROUTINES ====================

    // event handler called whenever new pipeline source is added to server manager
    void SourceAdded(pqPipelineSource* ptrPipelineSource);

    // event handler called whenever new data representation is added to pipeline source
    void RepresentationAdded(pqPipelineSource* ptrPipelineSource, pqDataRepresentation* ptrDataRepresentation, int iSourcePort);

    // event handler called whenever new data representation is created by object builder
    void DataRepresentationCreated(pqDataRepresentation* ptrDataRepresentation);

    //////////////////// PRIVATE ////////////////////

private:

    //==================== PROPERTIES ====================

    // lines pipeline source
    pqPipelineSource* mptrPipelineSource;

    // lines representation
    pqDataRepresentation* mptrLinesRepresentation;

    // labels points representation
    pqDataRepresentation* mptrLabelsPointsRepresentation;

    // per-point labels color
    QList<QVariant> mPointLabelsColor;

    // per-point labels size
    int miPointLabelsSize;

    // per-point label's format
    QString mstrPointLabelFormat;

    // per point label's justification
    int miPointLabelJustification;

    // data label representation
    pqDataRepresentation* mptrDataLabelRepresentation;
};

#endif // EVENT_HANDLERS_H

